@@ -35,7 +35,7 @@ to a BIND readable text form.
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.10'
+__version__ = '0.11'
 
 
 import os
@@ -110,6 +110,18 @@ class BindTreeExport(object):
     finally:
         self.db_instance.EndTransaction()
     cooked_data = self.CookData(data)
+    zone_view_assignments = {}
+    for zone_view_assignment in data['zone_view_assignments']:
+      if( not zone_view_assignment['zone_view_assignments_zone_name']
+              in zone_view_assignments):
+        zone_view_assignments[zone_view_assignment[
+            'zone_view_assignments_zone_name']] = []
+      zone_view_assignments[zone_view_assignment[
+          'zone_view_assignments_zone_name']].append(zone_view_assignment[
+          'zone_view_assignments_view_dependency'].split('_dep')[0])
+    for zone_view_assignment in zone_view_assignments:
+      if( zone_view_assignments[zone_view_assignment] == [u'any'] ):
+        raise Error('Zone "%s" has no view assignments.' % zone_view_assignment)
 
     record_arguments = data['record_arguments']
     record_argument_definitions = self.ListRecordArgumentDefinitions(
@@ -117,7 +129,11 @@ class BindTreeExport(object):
     for dns_server_set in cooked_data:
       config_parser = ConfigParser.SafeConfigParser()
       ## Make Files
-      dns_server_set_directory = '%s/%s_servers' % (self.root_config_dir,
+      named_directory = '%s/%s_servers' % (self.root_config_dir,
+          dns_server_set)
+      if( not os.path.exists(named_directory) ):
+        os.makedirs(named_directory)
+      dns_server_set_directory = '%s/%s_servers/named' % (self.root_config_dir,
           dns_server_set)
       if( not os.path.exists(dns_server_set_directory) ):
         os.makedirs(dns_server_set_directory)
@@ -134,19 +150,21 @@ class BindTreeExport(object):
         if( not os.path.exists(view_directory) ):
           os.makedirs(view_directory)
         for zone in cooked_data[dns_server_set]['views'][view]['zones']:
+          if( view not in zone_view_assignments[zone] ):
+            continue
           zone_file = '%s/%s/%s.db' % (dns_server_set_directory, view, zone)
           zone_file_string = zone_exporter_lib.MakeZoneString(
               cooked_data[dns_server_set]['views'][view]['zones'][zone][
                   'records'],
               cooked_data[dns_server_set]['views'][view]['zones'][zone][
                   'zone_origin'],
-              record_argument_definitions)
+              record_argument_definitions, zone, view)
           zone_file_handle = open(zone_file, 'w')
           try:
             zone_file_handle.writelines(zone_file_string)
           finally:
             zone_file_handle.close()
-      named_conf_file = '%s/named.conf' % dns_server_set_directory
+      named_conf_file = '%s/named.conf' % named_directory
       named_conf_file_string = self.MakeNamedConf(data, cooked_data,
                                                   dns_server_set)
       named_conf_file_handle = open(named_conf_file, 'w')
@@ -193,8 +211,12 @@ class BindTreeExport(object):
     """
     acl_dict = {}
     named_conf_lines = ['#This named.conf file is autogenerated. DO NOT EDIT']
-    named_conf_lines.append(self.ListLatestNamedConfGlobalOptions(
-        data, dns_server_set))
+    named_conf_header = self.ListLatestNamedConfGlobalOptions(data, dns_server_set)
+    if( not named_conf_header ):
+      raise Error('Named conf global options missing for server set "%s"' % (
+          dns_server_set))
+    named_conf_lines.append(named_conf_header)
+    named_conf_lines.extend(['options {', '  directory "/etc/named";', '};'])
     for acl in data['acls']:
       if( not acl['acl_name'] in acl_dict ):
         acl_dict[acl['acl_name']] = {}
@@ -230,7 +252,8 @@ class BindTreeExport(object):
                   'zone_origin'].rstrip('.')))
           named_conf_lines.append('\t\ttype %s;' % cooked_data[
               dns_server_set]['views'][view_name]['zones'][zone]['zone_type'])
-          named_conf_lines.append('\t\tfile "%s/%s.db";' % (view_name, zone))
+          named_conf_lines.append('\t\tfile "/etc/named/%s/%s.db";' % (
+              view_name, zone))
           zone_options = cooked_data[dns_server_set]['views'][view_name][
               'zones'][zone]['zone_options'].replace('\n', '\n\t\t')
           named_conf_lines.append('\t\t%s' % zone_options.rsplit('\n\t\t', 1)[0])
@@ -448,7 +471,7 @@ class BindTreeExport(object):
                 zone_name = zone['zone_view_assignments_zone_name']
                 if( view_dependency_name == zone[
                       'zone_view_assignments_view_dependency'] and
-                         (zone_name, view_dependency_name) in sorted_records ):
+                      (zone_name, view_dependency_name) in sorted_records ):
                   if( not view_name in cooked_data[
                         dns_server_set_name]['views'] ):
                     cooked_data[dns_server_set_name]['views'][view_name] = {}
@@ -461,14 +484,17 @@ class BindTreeExport(object):
                     cooked_data[dns_server_set_name]['views'][view_name][
                         'zones'] = {}
                   if( not zone_name in cooked_data[
-                        dns_server_set_name]['views'][view_name] ):
+                        dns_server_set_name]['views'][view_name]['zones'] ):
                     cooked_data[dns_server_set_name]['views'][view_name][
                         'zones'][zone_name]= {}
+                  if( 'records' not in cooked_data[dns_server_set_name][
+                          'views'][view_name]['zones'][zone_name] ):
+                      cooked_data[dns_server_set_name]['views'][view_name][
+                          'zones'][zone_name]['records'] = []
 
                   cooked_data[dns_server_set_name]['views'][view_name][
-                      'zones'][zone_name]['records'] = sorted_records[(
-                          zone_name, view_dependency_name)].values()
-
+                      'zones'][zone_name]['records'].extend(sorted_records[(
+                          zone_name, view_dependency_name)].values())
                   cooked_data[dns_server_set_name]['views'][view_name][
                       'zones'][zone_name]['zone_origin'] = zone['zone_origin']
 
